;!function(){try{var k="_btDebugIds",u="undefined",v="69bce514-9307-f2e1-dbc3-dd0150f7c3dc",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
import { jsonc } from "jsonc";
import { LIBRARY_ERR } from "../../../app/error/error-codes.js";
import { iCPSError } from "../../../app/error/error.js";
import { STASH_DIR } from "../constants.js";
/**
 * Potential AlbumTypes
 */
export var AlbumType;
(function (AlbumType) {
    AlbumType[AlbumType["FOLDER"] = 3] = "FOLDER";
    AlbumType[AlbumType["ALBUM"] = 0] = "ALBUM";
    AlbumType[AlbumType["ARCHIVED"] = 99] = "ARCHIVED";
})(AlbumType || (AlbumType = {}));
/**
 * This class represents a photo album within the library
 */
export class Album {
    /**
     * UUID of this album
     */
    uuid;
    /**
     * Album type of this album
     */
    albumType;
    /**
     * The name of this album
     */
    albumName;
    /**
     * Assets, where the key is the uuid & the value the filename
     */
    assets;
    /**
     * UUID of parent folder
     */
    parentAlbumUUID;
    /**
     * Constructs a new album
     * @param uuid - The UUID of the album
     * @param albumType - The album type of the album
     * @param albumName - The album name of the album
     * @param parentAlbumUUID - The UUID of the parent album
     */
    constructor(uuid, albumType, albumName, parentAlbumUUID) {
        this.uuid = uuid;
        this.albumType = albumType;
        this.albumName = albumName;
        this.parentAlbumUUID = parentAlbumUUID;
        this.assets = {};
    }
    /**
     *
     * @returns The display name of this album instance
     */
    getDisplayName() {
        return this.albumName;
    }
    /**
     *
     * @returns A valid filename, that will be used to store the album on disk
     */
    getSanitizedFilename() {
        return this.albumName.replaceAll(`/`, `_`);
    }
    /**
     *
     * @returns The UUID of this album instance
     */
    getUUID() {
        return this.uuid;
    }
    /**
     * Creates an album from a CPLAlbum instance (as returned from the backend)
     * @param cplAlbum - The album retrieved from the backend
     * @returns An Album based on the CPL object
     */
    static fromCPL(cplAlbum) {
        const album = new Album(cplAlbum.recordName, cplAlbum.albumType, Buffer.from(cplAlbum.albumNameEnc, `base64`).toString(`utf8`), cplAlbum.parentId ? cplAlbum.parentId : ``);
        album.assets = cplAlbum.assets ?? {};
        return album;
    }
    /**
     * Creates a dummy root album that is used to load all other albums from disk
     * @returns The dummy root album
     */
    static getRootAlbum() {
        return new Album(``, AlbumType.FOLDER, `iCloud Photos Library`, ``);
    }
    /**
     * Creates a dummy stash album tat is used to load all albums currently within the stash album
     * @returns The dummy stash album
     */
    static getStashAlbum() {
        return new Album(STASH_DIR, AlbumType.FOLDER, `iCloud Photos Library Archive`, ``);
    }
    /**
     *
     * @param album - An album to compare to this instance
     * @returns True if provided album is equal to this instance (based on UUID, AlbumType, AlbumName, Parent UUID & list of associated assets) - In case either of the albumTypes is archived, the list of assets will be ignored
     */
    equal(album) {
        return album
            && this.uuid === album.uuid
            && this.getSanitizedFilename() === album.getSanitizedFilename()
            && this.parentAlbumUUID === album.parentAlbumUUID
            && ( // If any of the albumTypes is ARCHIVED we will ignore asset and albumType equality
            this.albumType === AlbumType.ARCHIVED || album.albumType === AlbumType.ARCHIVED
                || (this.assetsEqual(album.assets)
                    && this.albumType === album.albumType));
    }
    /**
     * Checks of the assets, attached to this album are equal to the provided assets
     * @param assets - The list of assets to compare to
     * @returns True, if the assets are equal (order does not matter)
     */
    assetsEqual(assets) {
        // Assets might be undefined
        const thisAssets = this.assets ? this.assets : {};
        const otherAssets = assets ? assets : {};
        return jsonc.stringify(Object.keys(thisAssets).sort()) === jsonc.stringify(Object.keys(otherAssets).sort());
    }
    /**
     * Should only be called on a 'remote' entity. Will apply the local entity's properties to the remote one
     * @param localEntity - The local entity
     * @returns This object with the applied properties
     */
    apply(localEntity) {
        if (!localEntity) {
            return this;
        }
        this.albumType = localEntity.albumType;
        return this;
    }
    /**
     * Check if a given album is in the chain of ancestors
     * @param potentialAncestor - The potential ancestor for the given album
     * @param fullQueue - The full list of albums
     * @returns True if potentialAncestor is part of this album's directory tree
     */
    hasAncestor(potentialAncestor, fullQueue) {
        if (this.parentAlbumUUID === ``) { // If this is a root album, the potentialAncestor cannot be a ancestor
            return false;
        }
        if (potentialAncestor.getUUID() === this.parentAlbumUUID) { // If the ancestor is the parent, return true
            return true;
        }
        // Find actual parent
        const parent = fullQueue.find(album => album.getUUID() === this.parentAlbumUUID);
        // If there is a parent, check if it has the ancestor
        if (parent) {
            return parent.hasAncestor(potentialAncestor, fullQueue);
        }
        // If there is no parent, this means the queue has a gap and all we can assume is, that the ancestor is false
        return false;
    }
    /**
     * Calculates the distance to the root folder in order to compare the album's order.
     * @param album - The album, whose depth needs to be calculated
     * @param fullQueue - The list of all albums
     * @returns The number of albums between the given album and the root album
     * @throws An iCPSError, in case there is no link from the given album to root
     */
    static distanceToRoot(album, fullQueue) {
        if (album.parentAlbumUUID === ``) {
            return 0;
        }
        const parent = fullQueue.find(potentialParent => potentialParent.getUUID() === album.parentAlbumUUID);
        if (parent) {
            return Album.distanceToRoot(parent, fullQueue) + 1;
        }
        throw new iCPSError(LIBRARY_ERR.NO_DISTANCE_TO_ROOT);
    }
}
//# sourceMappingURL=album.js.map
//# debugId=69bce514-9307-f2e1-dbc3-dd0150f7c3dc